########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 18/05/2009
# Modified on 	: 20/05/2009 (Change of script names, procedure names and file names to make alternative methods clearer)
#				  20/05/2009 (Improved annotation and unsetting of variables)
#				  20/05/2009 (Addition of output file column for number of replication timing regions found per gene)
#				  20/05/2009 (Addition of output file column for chromosomal class)
#				  27/05/2009 (Changed column numbers so converted positions in build mm8 used)
#				  27/05/2009 (Changed input file name to use input file with lifted positions)
# 				  02/06/2009 (Control for sign of replication time when calculating difference in timing across genes)
#
# Input files 	: CombinedReplicationTables-chr.txt (where chr represents all mouse chromosomes)
#													(Output from the tcl script combine_replication_tables.tcl)
#				  K_genes.txt (Output from a combination of tcl scripts INTRONS*.tcl, specifically INTRONS16_Kgene.tcl)
# 
# Output files  : K_gene_RepTime_genic.txt (Contains substitution rate data plus new average, first and last replication times for each gene)
#
# This script uses the start and end locations of the coding sequence of each gene to identify replication timing regions for the gene.
# Replication timing regions that either COMPLETELY or PARTIALLY cover the gene are included.
# For each replication timing region, 4 data types are available. ALL of these are extracted and assigned to the gene.
# The average replication time for the gene is then determined from all data assigned to the gene, using both the MEAN and the MEDIAN.
# In addition, the average replication time closest to the start of the gene and closest to the end of the gene are extracted.
# The difference in these replication times is then calculated in order to determine how much variation there is across each gene compared to across the genome.
# All data for the orthologous gene (K, GC, refseq, positions etc) from K_gene.txt together with the new replication timing data is sent to the new file.
#
# Additional data regarding the number of replication timing regions assigned to the gene and the chromosomal class are also output.
#
# Script methodology:
#
# For each orthologous gene, it's start and end positions are extracted from K_gene.txt
# Replication timing regions for which data has been measured are indexed by their start position.
# For each replication start position on the chromosome on which the gene is located, the start and end positions of the region are identified.
# A comparison of the start and end positions of the replication region compared to the start and end positions of the gene determines whether
# the replication window is located completely or partially within the gene.
# If the replication region is completely or partially located within the gene, the replication start position is recorded.
# Once the position of all replication regions have been compared to the position of the gene, it is determined whether the gene encompased any
# replication timing regions.
# If so, replication timing data for each recorded replication start position is identified.
# Once all replication timing data (from the 4 cell lines) across the entire gene has been extracted, the averages of this data are calculated.
# These averages are then considered to be the replication time for the gene.
# Genes with no replication timing data are not included in the final data set.
# 
#				    Gene start															Gene end
#						|																	|
# o-------------------------------------------------------------------------------------------------------------------->
#				|	         |				|           |			|              |		       |            |
#				s1  	     e1     	    s2         e2	       s3             e3             s4            e4
#				|____________|				|___________|			|______________|		       |____________|
#				Partially included		  Completely included		Completely included				Not included
#
########################################################################################################################################################

package provide 22_allocate_RepTime_Kgenes_genic 1.0

proc 22_allocate_RepTime_Kgenes_genic {} {

	puts "-----------------------------------------------------------"
	puts "22   ASSIGN GENIC REP TIMES TO GENIC SUBSTITUTION RATES    \n"

	# Make variables global
	global contents
	global mean
	global median
	
	# Extract replication times for each chromosome
	puts "Extracting replication times for each chromosome and saving in arrays"
	cd Output_Files 
	foreach chromosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 X Y} {

		openfile CombinedReplicationTables-$chromosome.txt
		set contents [split $contents \n]
		set contents [lrange $contents 1 end-1]
		foreach dataline $contents {
			
			set GeneID [lindex $dataline 0]
			set StartPosition [lindex $dataline 2]
			set EndPosition [lindex $dataline 3]
			set Value_46C [lindex $dataline 4]
			set Value_D3 [lindex $dataline 5]
			set Value_iPS [lindex $dataline 6]
			set Value_TT2 [lindex $dataline 7]

			lappend STARTS($chromosome) $StartPosition
			set LOCATIONS($chromosome,$StartPosition) "$StartPosition $EndPosition"
			set DETAILS($chromosome,$StartPosition) "$StartPosition\t$EndPosition\t$Value_46C\t$Value_D3\t$Value_TT2\t$Value_iPS"

			unset GeneID StartPosition EndPosition Value_46C Value_D3 Value_iPS Value_TT2
		}
		unset dataline contents
	}
	unset chromosome
	cd ..
	puts "Replication times for each chromosome extracted to arrays"

	

	# Initiate output files
	set outTXT [open K_gene_RepTime_genic.txt w]
	set outXLS [open K_gene_RepTime_genic.xls w]
	
	puts $outTXT "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_Replication_Regions\tMean_RT\tMedian_RT\tMean_RT_start\tMean_RT_end\tMean_RT_difference\tMedian_RT_start\tMedian_RT_end\tMedian_RT_difference"
	puts $outXLS "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_Replication_Regions\tMean_RT\tMedian_RT\tMean_RT_start\tMean_RT_end\tMean_RT_difference\tMedian_RT_start\tMedian_RT_end\tMedian_RT_difference"
	

	# Assign genic replication times to genic substitution rates	
	puts "Assigning genic replication times to genic substitution rates"


	cd Output_Files
	openfile K_gene_liftOver.txt
	cd ..
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	set count 0
	foreach dataline $contents {
		
		incr count
		if {[expr {$count %500}] == 0} {puts "Gene $count of [llength $contents]"}

		
		# Extract gene details
		set ortholog [lindex $dataline 0]
		set chromosome [lindex $dataline 1]
		set startGENE [lindex $dataline 8]
		set endGENE [lindex $dataline 9]
		
		# Determine chromosomal class
		if {$chromosome == "X"} {
			set class X
		} elseif {$chromosome == "Y"} {
			set class Y
		} else {
			set class A
		}
		
		# Initiate variables to store replication timing start positions and data at the start, end and over the entire gene
		set replicationSTARTS {}
		set replicationtimeFIRST {}
		set replicationtimeLAST {}
		set replicationtimeALL {}

		# Set variable used to determine if any replication data has been found for this gene
		set foundRT 0

		# FIND REGIONS SPANNING GENE
		# Generate a list of all replication starts with data for this gene and store in variable replicationSTARTS
		foreach replicationstart $STARTS($chromosome) {
			
			set startREPLICATION [lindex $LOCATIONS($chromosome,$replicationstart) 0]
			set endREPLICATION [lindex $LOCATIONS($chromosome,$replicationstart) 1]
			
			# Replication timing regions COMPLETELY within the gene
			if {$startREPLICATION >= $startGENE && $endREPLICATION <= $endGENE } {
				lappend replicationSTARTS $replicationstart
				incr foundRT
			}
			# Replication timing regions PARTIALLY covering the start of the gene
			if {$startGENE >= $startREPLICATION && $startGENE <= $endREPLICATION} {
				lappend replicationSTARTS $replicationstart
				incr foundRT
			}
			# Replication timing regions PARTIALLY covering the end of the gene
			if {$endGENE >= $startREPLICATION && $endGENE <= $endREPLICATION} {
				lappend replicationSTARTS $replicationstart
				incr foundRT
			}
					
			unset startREPLICATION endREPLICATION
			
		}
		unset replicationstart
		

		# EXTRACT REPLICATION TIMES FOR THESE REGIONS
		# Extract start, end and all replication times if any replication regions have been found to cover this gene
		if {$foundRT > 0} {

			# Sort replication timing regions for the gene into assending order to identify first and last regions
			set replicationSTARTS [lsort -integer $replicationSTARTS]
			
			set first [lindex $replicationSTARTS 0]
			set last [lindex $replicationSTARTS end]
			
			lappend replicationtimeFIRST [lindex $DETAILS($chromosome,$first) 2]
			lappend replicationtimeFIRST [lindex $DETAILS($chromosome,$first) 3]
			lappend replicationtimeFIRST [lindex $DETAILS($chromosome,$first) 4]
			lappend replicationtimeFIRST [lindex $DETAILS($chromosome,$first) 5]
			
			lappend replicationtimeLAST [lindex $DETAILS($chromosome,$last) 2]
			lappend replicationtimeLAST [lindex $DETAILS($chromosome,$last) 3]
			lappend replicationtimeLAST [lindex $DETAILS($chromosome,$last) 4]
			lappend replicationtimeLAST [lindex $DETAILS($chromosome,$last) 5]

			foreach repstart $replicationSTARTS {
				
				lappend replicationtimeALL [lindex $DETAILS($chromosome,$first) 2]
				lappend replicationtimeALL [lindex $DETAILS($chromosome,$first) 3]
				lappend replicationtimeALL [lindex $DETAILS($chromosome,$first) 4]
				lappend replicationtimeALL [lindex $DETAILS($chromosome,$first) 5]
			
			}
			unset repstart
			unset first last

			# Calculate mean replication times for the start, end and entire gene
			calculateMean $replicationtimeFIRST
			set meanFIRST $mean
			unset mean
			
			calculateMean $replicationtimeLAST
			set meanLAST $mean
			unset mean
			
			calculateMean $replicationtimeALL
			set meanALL $mean
			unset mean

			# Calculate median replication times for the start, end and entire gene
			calculateMedian $replicationtimeFIRST
			set medianFIRST $median
			unset median
			
			calculateMedian $replicationtimeLAST
			set medianLAST $median
			unset median
			
			calculateMedian $replicationtimeALL
			set medianALL $median
			unset median


			# Determine difference if both replication times negative
			if {$meanFIRST < 0 && $meanLAST < 0} {
				if {$meanLAST > $meanFIRST} {set differenceMEAN [expr {$meanLAST - $meanFIRST}]} else {set differenceMEAN [expr {$meanFIRST - $meanLAST}]}
			# Determine difference if both replication times positive
			} elseif {$meanFIRST > 0 && $meanLAST > 0} {
				if {$meanLAST > $meanFIRST} {set differenceMEAN [expr {$meanLAST - $meanFIRST}]} else {set differenceMEAN [expr {$meanFIRST - $meanLAST}]}
			# Determine difference if replication times are positive and negative
			} else {
				if {$meanFIRST < 0} {set differenceMEAN [expr {[expr {$meanFIRST * -1}] + $meanLAST}]} else {set differenceMEAN [expr {[expr {$meanLAST * -1}] + $meanFIRST}]}
			}

			# Determine difference if both replication times negative
			if {$medianFIRST < 0 && $medianLAST < 0} {
				if {$medianLAST > $medianFIRST} {set differenceMEDIAN [expr {$medianLAST - $medianFIRST}]} else {set differenceMEDIAN [expr {$medianFIRST - $medianLAST}]}
			# Determine difference if both replication times positive
			} elseif {$medianFIRST > 0 && $medianLAST > 0} {
				if {$medianLAST > $medianFIRST} {set differenceMEDIAN [expr {$medianLAST - $medianFIRST}]} else {set differenceMEDIAN [expr {$medianFIRST - $medianLAST}]}
			# Determine difference if replication times are positive and negative
			} else {
				if {$medianFIRST < 0} {set differenceMEDIAN [expr {[expr {$medianFIRST * -1}] + $medianLAST}]} else {set differenceMEDIAN [expr {[expr {$medianLAST * -1}] + $medianFIRST}]}
			}

			
			# Send original data for the gene and new replication time data to output files		
			puts $outTXT "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t[lindex $dataline 20]\t$class\t[llength $replicationSTARTS]\t$meanALL\t$medianALL\t$meanFIRST\t$meanLAST\t$differenceMEAN\t$medianFIRST\t$medianLAST\t$differenceMEDIAN"
			puts $outXLS "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t[lindex $dataline 20]\t$class\t[llength $replicationSTARTS]\t$meanALL\t$medianALL\t$meanFIRST\t$meanLAST\t$differenceMEAN\t$medianFIRST\t$medianLAST\t$differenceMEDIAN"
	
			# Unset variables
			unset meanFIRST meanLAST meanALL medianFIRST medianLAST medianALL
			unset differenceMEAN differenceMEDIAN
	
		}

		# Unset variables
		unset ortholog chromosome startGENE endGENE
		unset replicationSTARTS replicationtimeFIRST replicationtimeLAST replicationtimeALL
		unset foundRT
		unset class

	}
	unset dataline contents

	puts "Genic replication times assigned to genic substitution rates"


	# Close output file and move to Output_Files directory
	close $outTXT
	close $outXLS
	unset outTXT outXLS
	
	exec mv K_gene_RepTime_genic.txt Output_Files/K_gene_RepTime_genic.txt
	exec mv K_gene_RepTime_genic.xls Output_Files/K_gene_RepTime_genic.xls


	# Unset variables
	unset STARTS LOCATIONS DETAILS

	puts "\nVariables open in 04_allocate_RepTime_Kgenes_genic : [info local]"

return 
}
