########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 15/05/2009
# Modified on 	: 18/05/2009 (Find replication time for the gene and assign it to the intron)
# 				  20/05/2009 (Change of script names, procedure names and file names to make alternative methods clearer)
#				  20/05/2009 (Tidying of script, removal of chromosomal counts and timing)
#				  20/05/2009 (Actually assign replication rates and send to output file! - Use complete & partical coverage method)
#				  20/05/2009 (Removal of counts of alternative complete and partial coverage methods)
#				  20/05/2009 (Allocation of replication times using wider 'partial' coverage method)
#				  01/06/2009 (Changed column numbers so converted positions in build mm8 used)
#				  01/06/2009 (Changed input file name to use input file with lifted positions)
#
# Input files 	: CombinedReplicationTables-chr.txt (where chr represents all mouse chromosomes)
#													(Output from the tcl script combine_replication_tables.tcl)
#				  K_introns.txt (Output from a combination of tcl scripts INTRONS*.tcl, specifically INTRONS16_Kintron.tcl)
# 
# Output files  : K_intron_RepTime_intronic.txt (Contains substitution rate data plus new chromosomal class and average replication timing data for each intron)
#
# This script uses the start and end locations of each intron to identify replication timing regions for the intron.
# Replication timing regions that either COMPLETELY or PARTIALLY cover the intron are included.
# For each replication timing region, 4 data types are available. ALL of these are extracted and assigned to the intron.
# The average replication time for the intron is then determined from all data assigned to the intron, using both the MEAN and the MEDIAN.
# All data for the orthologous intron (K, GC, refseq, positions etc) from K_intron.txt together with the new replication timing data is sent to the new file.
# Additional data regarding the number of replication timing regions assigned to the intron and the chromosomal class are also output.
#
# Script methodology:
#
# For each orthologous intron, it's start and end positions are extracted from K_intron.txt
# Replication timing regions for which data has been measured are indexed by their start position.
# For each replication start position on the chromosome on which the intron is located, the start and end positions of the region are identified.
# A comparison of the start and end positions of the replication region compared to the start and end positions of the intron determines whether
# the replication window is located completely or partially within the intron.
# If the replication region is completely or partially located within the intron, the replication start position is recorded.
# Once the position of all replication regions have been compared to the position of the intron, it is determined whether the intron encompased any
# replication timing regions.
# If so, replication timing data for each recorded replication start position is identified.
# Once all replication timing data (from the 4 cell lines) across the entire intron has been extracted, the averages of this data are calculated.
# These averages are then considered to be the replication time for the intron.
# Introns with no replication timing data are not included in the final data set.
# 
#				   Intron start															Intron end
#						|																	|
# o-------------------------------------------------------------------------------------------------------------------->
#				|	         |				|           |			|              |		       |            |
#				s1  	     e1     	    s2         e2	       s3             e3             s4            e4
#				|____________|				|___________|			|______________|		       |____________|
#				Partially included		  Completely included		Completely included				Not included
#
########################################################################################################################################################

package provide 23_allocate_RepTime_Kintrons_intronic 1.0

proc 23_allocate_RepTime_Kintrons_intronic {} {

	puts "-----------------------------------------------------------"
	puts "23 ASSIGN INTRONIC REP TIMES TO INTRONIC SUBSTITUTION RATES\n"

	# Make variables global
	global contents
	global mean
	global median
	
	# Extract replication times for each chromosome
	puts "Extracting replication times for each chromosome and saving in arrays"
	cd Output_Files 
	foreach chromosome {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 X Y} {

		openfile CombinedReplicationTables-$chromosome.txt
		set contents [split $contents \n]
		set contents [lrange $contents 1 end-1]
		foreach dataline $contents {
			
			set GeneID [lindex $dataline 0]
			set StartPosition [lindex $dataline 2]
			set EndPosition [lindex $dataline 3]
			set Value_46C [lindex $dataline 4]
			set Value_D3 [lindex $dataline 5]
			set Value_iPS [lindex $dataline 6]
			set Value_TT2 [lindex $dataline 7]

			lappend STARTS($chromosome) $StartPosition
			set LOCATIONS($chromosome,$StartPosition) "$StartPosition $EndPosition"
			set DETAILS($chromosome,$StartPosition) "$StartPosition\t$EndPosition\t$Value_46C\t$Value_D3\t$Value_TT2\t$Value_iPS"

			unset GeneID StartPosition EndPosition Value_46C Value_D3 Value_iPS Value_TT2
		}
		unset dataline contents
	}
	unset chromosome
	cd ..
	puts "Replication times for each chromosome extracted to arrays"




	# Initiate output files
	set outTXT [open K_intron_RepTime_intronic.txt w]
	set outXLS [open K_intron_RepTime_intronic.xls w]
	
	puts $outTXT "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_RT_regions\tMean_RT\tMedian_RT"
	puts $outXLS "Ortholog\tIntron_Number\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tMouse_Start_mm9\tMouse_End_mm9\tMouse_Start_mm8\tMouse_End_mm8\tRat_Start\tRat_End\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC\tClass\tNumber_RT_Regions\tMean_RT\tMedian_RT"


	# Assign genic replication times to genic substitution rates	
	puts "Assigning intronic replication times to intronic substitution rates"

	cd Output_Files
	openfile K_intron_liftOver.txt
	cd ..
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	set count 0
	foreach dataline $contents {
		
		incr count
		if {[expr {$count %100}] == 0} {puts "*** $count of [llength $contents] ***"}

		
		# Extract chromosome and intronic position for this intron
		set chromosome [lindex $dataline 2]
		set startINTRON [lindex $dataline 8]
		set endINTRON [lindex $dataline 9]
		
		# Determine chromosomal class
		if {$chromosome == "X"} {
			set class X
		} elseif {$chromosome == "Y"} {
			set class Y
		} else {
			set class A
		}

		# Initiate variables to store replication timing start positions
		set replicationSTARTS {}

		# Set variable used to determine if any replication data has been found for this intron
		set foundRT 0
			
		# FIND REGIONS SPANNING INTRON
		# Generate a list of all replication starts with data for this intron and store in variable replicationSTARTS
		foreach replicationstart $STARTS($chromosome) {
			
			set startREPLICATION [lindex $LOCATIONS($chromosome,$replicationstart) 0]
			set endREPLICATION [lindex $LOCATIONS($chromosome,$replicationstart) 1]
			
			# Replication timing regions COMPLETELY within the intron
			if {$startREPLICATION >= $startINTRON && $endREPLICATION <= $endINTRON } {
				incr foundRT
				lappend replicationSTARTS $replicationstart
			}
			# Replication timing regions PARTIALLY covering the start of the intron
			if {$startINTRON >= $startREPLICATION && $startINTRON <= $endREPLICATION} {
				incr foundRT
				lappend replicationSTARTS $replicationstart
			}
			# Replication timing regions PARTIALLY covering the end of the intron
			if {$endINTRON >= $startREPLICATION && $endINTRON <= $endREPLICATION} {
				incr foundRT
				lappend replicationSTARTS $replicationstart				
			}
					
			unset startREPLICATION endREPLICATION
			
		}
		unset replicationstart
				
		
		# EXTRACT REPLICATION TIMES FOR THESE REGIONS
		# Extract all replication times if any replication regions have been found to cover this intron

		if {$foundRT > 0} {
			
			set replicationDATA {}

			foreach repstart $replicationSTARTS {
				lappend replicationDATA [lindex $DETAILS($chromosome,$repstart) 2]
				lappend replicationDATA [lindex $DETAILS($chromosome,$repstart) 3]
				lappend replicationDATA [lindex $DETAILS($chromosome,$repstart) 4]
				lappend replicationDATA [lindex $DETAILS($chromosome,$repstart) 5]
			}
			
			# Calculate average replication times for the intron from all replication timing data covering any part of the intron
			calculateMean $replicationDATA
			set meanRT $mean
			unset mean
			
			calculateMedian $replicationDATA
			set medianRT $median
			unset median

			# Send original data and new average replication times to output file			
			puts $outTXT "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t$class\t[llength $replicationSTARTS]\t$meanRT\t$medianRT"
			puts $outXLS "[lindex $dataline 0]\t[lindex $dataline 1]\t[lindex $dataline 2]\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]\t[lindex $dataline 11]\t[lindex $dataline 12]\t[lindex $dataline 13]\t[lindex $dataline 14]\t[lindex $dataline 15]\t[lindex $dataline 16]\t[lindex $dataline 17]\t[lindex $dataline 18]\t[lindex $dataline 19]\t$class\t[llength $replicationSTARTS]\t$meanRT\t$medianRT"
			
			unset replicationDATA repstart meanRT medianRT
		}
	
		# Unset variables
		unset chromosome startINTRON endINTRON
		unset foundRT
		unset replicationSTARTS
		unset class
		
	}
	unset dataline contents

	puts "Intronic replication times assigned to intronic substitution rates"


	# Close output file and move to Output_Files directory
	close $outTXT
	close $outXLS
	unset outTXT outXLS 

	exec mv K_intron_RepTime_intronic.txt Output_Files/K_intron_RepTime_intronic.txt
	exec mv K_intron_RepTime_intronic.xls Output_Files/K_intron_RepTime_intronic.xls



	# Unset variables
	unset STARTS LOCATIONS DETAILS

	puts "\nVariables open in 05_allocate_RepTime_Kintrons_intronic : [info local]"

return
}
