########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 16/11/2007 (As part of MMBIntrons work)
# Modified on 	: 08/05/2009 (Change of directory names and script structure to fit in with current analysis)
#							 (Removal of counts to be output to information file)
# 						     (Deletion of unwanted files rather than storage in new directories as checking done as part of MMB analysis and to reduce file number)
#							 (Inclusion of position of introns in intron annotation)
#				  28/05/2009 (Extraction of limits of coding sequence to use as genic positions)
#				  29/05/2009 (Extraction of all exonic and intronic positions to file for liftOver)
#
# Input files 	: NM_*****.txt for each gene stored in directories GENES_Organism
#				  (Output from the tcl script "INTRONSextractgenes.tcl")
#					
# Output files  : IN_NM_*****.txt for each gene stored in directories INTRONS_Organism
#				  GeneData_Organism.txt for each organism containing details of exon number, phase, location to determine true orthologs
#				  POSITIONS-$organism.txt for each organism containing details of each intron and exon position
#				  CDSpositions-$organism.txt for each organism containing details of each exon position
#
# Procedure to separate out introns for each gene
# From each organisms GENES_$organism directory, identify all the gene files present
# Create directories to store each organisms introns (INTRONS_$organism) in
# For each gene file, identify the genes refseqID and then separate the different gene regions from the file
# For each region, split into annotation and sequence and determine if the sequence is exonic (UPPERCASE) or intronic (lowercase)
# NB. Due to the long intronic sequence lengths, it is faster to separate the annotation from the sequence using the [split $string \n] command than to use regexp to each part of the string
# Determine the exon phase and exon length of exons and add this information to the exon annotation.
# Determine the start and end position of each exon and append to a list of all positions for each refseq gene.
# Output the regions annotation and sequence to the relevant intronic (IN_$refseqID.txt) output file, referenced by the RefSeq gene ID
# Determine if the region contains introns and is therefore suitable for the purposes of the analysis
# If the gene contains introns, send output files to intron directories (IN_$refseqID.txt to INTRONS_$organism)
# For genes containing introns, send the refseq gene ID, chromosome, number of introns, number of exons, exon phases, exon lengths and intron lengths to output file GeneData_$organism.txt so that it can be used for true orthologue identification
# If the gene doesn't contain introns, eliminate the gene from the analysis by deleting the file.
# If the output file contains an exon, delete the file
# For each refseq gene, find the minimum and maximum positions and save in output file as the limits of the coding sequence for the gene.
#
########################################################################################################################################################

package provide INTRONS02_extractintrons 1.0

proc INTRONS02_extractintrons {} {
    
	puts "-----------------------------------------------------------"
	puts "02            EXTRACT INTRONS FROM GENE FILES              \n"

    # Make variables global
    global contents
    
    # Repeat procedure for each organisms genes
	cd Intron_Data
    set GeneDirectories [glob GENES_*]

    foreach GeneDirectory $GeneDirectories {
	
		# Determine organism
		regexp {GENES_(.+)} $GeneDirectory all organism
		puts "Separating $organism introns from exons"
	
		# Create directories to store organisms exons and introns
		if {![file exists INTRONS_$organism]} {
	   		puts "No INTRONS_$organism directory was found"
	    	exec mkdir INTRONS_$organism
	    	puts "New INTRONS_$organism directory created"
		} elseif {[file exists INTRONS_$organism]} {
	    	exec rm -r INTRONS_$organism
	    	puts "INTRONS_$organism directory was found and removed"
	    	exec mkdir INTRONS_$organism
	    	puts "New INTRONS_$organism directory created"
		}
			
			
		set outPOSITIONS [open POSITIONS-$organism.txt w]
		puts $outPOSITIONS "RefSeq\tSequence\tChromosome\tStart\tEnd"
					
		# Create output file for exon data to enable identification of true orthologs
		set genedataout [open GeneData_$organism.txt w]
		puts $genedataout "RefseqID :\tChromosome :\tIntron Number :\tExon Number :\tExon Phases :\tExon Lengths :\tIntron Lengths"

	
		# Move to the relevant directory
		cd $GeneDirectory
		set filelist [glob *.txt]
		cd ..
		
		# Separate Exons and Introns from each file
		foreach fl $filelist {
	    
	    	# Identifly gene refseq ID
	    	regexp {(.+)\.txt} $fl all refseqID
	    
	    	# Separate file contents into regions
	    	cd $GeneDirectory
	    	openfile $fl
	    	cd ..
	    	set regions [split $contents >]
	    	set regions [lreplace $regions 0 0]
	    	unset contents
	    
	    	# Initiate counts of exons and introns
	    	set exoncount 0
	    	set introncount 0
	    
	    	# Initiate lists to store exon data in
	    	set exonlengths {}
	    	set exonphases {}
	    	set intronlengths {}
	    
	    	# Create output files for introns and exons
	    	set outExons [open EX_$refseqID.txt w]
	    	set outIntrons [open IN_$refseqID.txt w]
	    
	    	# For each region, determine if exon or intron and send to relevant output file
	    	foreach region $regions {
				# Split region into annotation and sequence to increase speed of regexp
				set splitregion [split $region \n]
				set regionannot [lindex $splitregion 0]
				set regionseq [lindex $splitregion 1]
				unset splitregion
		
				# Identify gene ID and chromosome from annotation	
				regexp {([a-z][a-z]).+refGene_(.+)\s.+=(chr[\w]+):([0-9]+)-([0-9]+).+} $regionannot all species geneID chromosome startpos endpos
				# If sequence is in uppercase, send exon to output file
				if {[regexp {[A-Z]} $regionseq all ]} {
					incr exoncount
		    		regsub -all {\n$} $regionseq "" regionseq
		    		# Determine the exon phase and length
		    		set exonlength [string length $regionseq]
		    		set exonphase [expr $exonlength % 3]
		    		# Append exon data to lists
		    		lappend exonlengths $exonlength
		    		lappend exonphases $exonphase
					puts $outExons ">$species:$geneID:$chromosome:$exonphase:$exonlength\n$regionseq"
		    		unset exonlength exonphase 
					
					# Output position details for region
					puts $outPOSITIONS "$geneID\tExon\t$chromosome\t$startpos\t$endpos"

					
					# Extract positions for exon
					if {[regexp {NM_[0-9]+_[0-9]+} $geneID all]} {
						regexp {(NM_[0-9]+)_[0-9]+} $geneID all refseq
						lappend EXONPOSITIONS($refseq) $startpos
						lappend EXONPOSITIONS($refseq) $endpos
						lappend GeneIDs $refseq
						unset all refseq
					} else {
						lappend EXONPOSITIONS($geneID) $startpos
						lappend EXONPOSITIONS($geneID) $endpos
						lappend GeneIDs $geneID
					}

				# If sequence is in lowercase, send intron to output file
				} elseif {[regexp {[a-z]} $regionseq all]} {
					incr introncount
		    		regsub -all {\n$} $regionseq "" regionseq
		    		puts $outIntrons ">$species:$geneID:$chromosome:$startpos-$endpos\n$regionseq"
		
					# Output position details for region
					puts $outPOSITIONS "$geneID\tIntron\t$chromosome\t$startpos\t$endpos"
		
		    		# Determine intron length and append to length
		    		set intronlength [string length $regionseq]
		    		lappend intronlengths $intronlength
		    		unset intronlength
				}
				# Unset variables
				unset species geneID regionannot regionseq startpos endpos
	    	}
	    
	    	# Close output files
	    	close $outExons
			close $outIntrons
	    
	    	# If the gene contains introns, send files to Exon and Intron directories
	    	if {$introncount > 0} {
				exec rm EX_$refseqID.txt 
				exec mv IN_$refseqID.txt INTRONS_$organism/IN_$refseqID.txt

				# Send exon datat to output file for identification of true orthologues
				puts $genedataout "$refseqID:$chromosome:$introncount:$exoncount:$exonphases:$exonlengths:$intronlengths"

	    	# If the gene doesn't contain introns, eliminate the gene from the analysis
	    	} elseif {$introncount == 0} {
				exec rm EX_$refseqID.txt
				exec rm IN_$refseqID.txt
	    	}	    	    
	    	# Unset variables
			unset refseqID region regions introncount exoncount exonphases exonlengths intronlengths chromosome outExons outIntrons
		
		}
	
		# Determine whether counted number of genes containing introns matches number of output files containing intron sequences
		cd INTRONS_$organism
		set filespresent [glob *.txt]
		cd ..
		

		# Close exon data output file
		close $genedataout
		unset genedataout
	
	
		# Identify coding sequence limits for each gene and output to a new file
		set outEXpos [open CDSpositions-$organism.txt w]
		puts $outEXpos "Refseq\tCDS_start\tCDS_end"
		set GeneIDs [lsort -unique $GeneIDs]
		foreach refseq $GeneIDs {
			set sortedpositions [lsort -integer $EXONPOSITIONS($refseq)]
			puts $outEXpos "$refseq\t[lindex $sortedpositions 0]\t[lindex $sortedpositions end]"
			unset sortedpositions
		}
		unset refseq GeneIDs
		close $outEXpos
		unset outEXpos
		
		# Close all positions output file
		close $outPOSITIONS
		unset outPOSITIONS
	
		# Unset variables
		unset organism fl filelist  filespresent 
		unset EXONPOSITIONS
		
		
	}
    unset GeneDirectory GeneDirectories
  	
	cd ..

    puts "\nVariables open in INTRONS02_extractintrons : [info local]"
       
return
}
