########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 21/11/2007 (As part of MMBIntrons work)
# Modified on 	: 08/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  11/05/2009 (Addition of code to create full conversion list from most recent flat files)
#				
# Input files 	: orthologsMGI_Entrez.txt (output by tcl script INTRONSorthologlist.tcl)
# 				  mouse.rna.gbff (downloaded from ftp://ftp.ncbi.nih.gov/refseq/M_musculus/mRNA_Prot/ and unzipped; dated 04/05/09)
# 				  rat.rna.gbff (downloaded from ftp://ftp.ncbi.nih.gov/refseq/R_norvegicus/mRNA_Prot/ and unzipped; dated 04/05/09)
#
# Output files  : Entrez2RS.txt (Full conversion list of rodent Entrez to RefSeq gene IDS)
#				  orthologsMGI_RefSeq.txt (List of orthologous RefSeq IDs)
#
# Gene (intron and exon) sequences downloaded from UCSC table browser are annotated by RefSeq ID. 
# However, the MGI database used to identify orthologous genes references gene by their Entrez ID.
# A conversion list is therefore required of comparable RefSeq and Entrez IDs for each gene.
# This conversion list is output as "Entrez2RS.tfa"
# Each time a conversion is done it is recommended that this script is re-run to convert a larger sample of genes.
#
# Extract list of pairs of orthologous gene names from ortholougesMGI_Entrez.tfa  file
# Extract pairs of mouse and rat Entrez gene IDs and their corresponding refseq gene ID from LDH file Entrez2RS.txt and set refseq IDs as an array: CONVERSION($organism,$Entrez) = $RefSeq
# Create a list of Entrez gene IDs that have a corresponding RefSeq ID in the conversion list
# Determine if each pair of orthologues Entrez gene IDs can be converted using the conversion list by searching the list of converted Entrez gene IDs
# If both the mouse and rat Entrez genes could be converted to RefSeq gene IDs, send the converted mouse and rat RefSeq gene IDS to output file orthologuesMGI_RefSeq.tfa
#
########################################################################################################################################################

package provide INTRONS04_convertgeneid 1.0

proc INTRONS04_convertgeneid {} {
	
	puts "-----------------------------------------------------------"
	puts "04   CONVERT ORTHOLOGOUS GENE IDS FROM ENTREZ TO REFSEQ    \n"
	
	# Set variables as global
	global contents
	
		
	
	# EXTRACT REFSEQID ACCESSION NUMBERS AND THEIR CORRESPONDING ENTREZ GENE IDS FROM FLAT FILES

	# Determine list of input refseq files (requires download from NCBI)
	cd Input_Files
	set flist [glob *.rna.gbff]
	cd ..
	
	# Create output file to store pairs of corresponding gene IDs for conversion
	set out [open Entrez2RS.txt w]
	puts $out "Species\tEntrez_id\tAccn"

	set countgenes 0
	set converted 0

	# Go through each organisms file in turn to extract gene IDs
	foreach fl $flist {
    
		puts "Extracting refseq & entrez gene IDs from $fl"

    	# Determine the species
    	regexp {([A-Za-z]+?)\.rna.gbff} $fl all sp
    
    	# Read input file
		cd Input_Files
    	openfile $fl
    	cd ..

    	# Insert a £ symbol before every locus, then split the file into gene annotations around £
    	regsub -all {LOCUS\s\s} $contents "£LOCUS  " contents
     	set contents [split $contents £]
  
    	# Work through each gene annotation in turn
    	foreach gene $contents {
			incr countgenes
	
			# Ensure the gene contains a codig sequence - only pull out protein encoding IDs
			if {[regexp {/db_xref="GeneID:[0-9]+?"} $gene]  && [regexp CDS $gene]} {
	    		incr converted
	    
	    		regexp {/db_xref="GeneID:([0-9]+?)"} $gene all LL
	    		regexp {LOCUS.*?(\S+?)\s} $gene all RS
	    	
	    		puts $out "$sp\t$LL\t$RS"
	    
	    		# Unset variables so they can't be reused
	    		unset all LL RS
			}    
   		}
    	# Unset variables
    	unset gene contents sp
	} 
	unset fl flist	
	
	# Close output file and move it to Intron_Data directory
	close $out
	unset out
	exec mv Entrez2RS.txt Intron_Data/Entrez2RS.txt
	puts "Gene ID conversion list created: Entrez2RS.txt"

	# Output summary data
	puts "Genes searched:                                   $countgenes"
	puts "Successfully converted:                           $converted"
	unset countgenes converted
	
	
	
	

	puts "Converting list of orthologous genes from Entrez to RefSeq"

	# OPEN LIST OF ORTHOLOGUE PAIR ENTREZ IDS
	
	# Create a list of sublists, each of which contains a pair of orthologue EntrezGene IDs
	cd Intron_Data
	openfile orthologsMGI_Entrez.txt
	cd ..
	set orthologuepairs [split $contents \n]
	set orthologuepairs [lreplace $orthologuepairs end end]
	unset contents
	
	# CREATE AN ARRAY OF THE GENE ID CONVERSIONS
	
	# Open the conversion file and extract the contents
	cd Intron_Data
	openfile Entrez2RS.txt
	cd ..
	# Split each gene ID conversion data into a separate sublist
	set conversionlist [split $contents \n]
	set conversionlist [lrange $conversionlist 1 end-1]
	unset contents
	
	# INITIATE LISTS TO STORE CONVERSION STATISTICS IN
	
	# Initiate a count of the number of orthologues dealt with
	set orthocount 0
	# Initiate a count of the number of orthologue pairs converted
	set refseqpaircount 0
	# Initiate lists of the Entrez IDS in the conversion list
	set EntrezList {}
	# Initiate counts of the number of gene IDs that are/are not converted
	set convertedmouse 0
	set convertedrat 0
	set notconvertedmouse 0
	set notconvertedrat 0
	# Initiate a list of orthologue genes that can't be converted for checking
	set nottheremouse {}
	set notthererat {}
	
	# Create a two dimensional array using the organism and EntrezGene ID as the index to identify the RefSeq code
	foreach conversion $conversionlist {
		# Etract the organism and gene IDs for the gene
		regexp {([a-z]+?)\s+?([\w]+)\s+?([A-Z][A-Z]_[0-9]+$)} $conversion all organism Entrez RefSeq
		# Set extracted information in an array
		set CONVERSION($organism,$Entrez) $RefSeq
		# Append the converted genes to the relevant lists
		lappend EntrezList $Entrez
		# Unset variables so they can't be reused
		unset organism Entrez RefSeq
	}
	unset conversion conversionlist
	
	# Create output file to store converted orthologue pair Gene IDs in
	set out [open orthologsMGI_RefSeq.txt w]
	
	# Go through each pair of orthologue genes and extract gene sequences
	foreach orfl $orthologuepairs {
		# Initiate counts to determine if both EntrezGene IDs have been converted
		set mouseconverted 0
		set ratconverted 0
		# Increase count of pairs dealt with to check it correlates with the number initially identified.
		incr orthocount
		# Identify each entrez gene in the orthologue pair
		regexp {([0-9]+)\s+?([0-9]+)} $orfl all mouseEntrez ratEntrez
		
		# Convert the orthologue pair EntrezGene IDs to RefSeq gene IDs
		# MOUSE
		if {[lsearch $EntrezList $mouseEntrez] >= 0} {
			set mouseRefSeq $CONVERSION(mouse,$mouseEntrez)
			incr convertedmouse
			incr mouseconverted
		} else {
			incr notconvertedmouse
			lappend nottheremouse $mouseEntrez
		}
		# RAT
		if {[lsearch $EntrezList $ratEntrez] >= 0} {
			set ratRefSeq $CONVERSION(rat,$ratEntrez)
			incr convertedrat
			incr ratconverted
		} else {
			incr notconvertedrat
			lappend notthererat $ratEntrez
		}
		# Determine if both EntrezGene IDs have been converted to RefSeq Gene IDs
		if {$mouseconverted > 0 & $ratconverted > 0} {
			# Send orthologue pair of RefSeq Gene IDs to output file
			puts $out "$mouseRefSeq\t$ratRefSeq"
			# Increase count of pairs of RefSeq IDs
			incr refseqpaircount
			unset mouseRefSeq ratRefSeq
		}
		# Unset variables
		unset mouseconverted ratconverted mouseEntrez ratEntrez
	}
	
	# Unset variables
	unset orfl orthologuepairs CONVERSION
	
	# Close output file and send to Intron_Data directory
	close $out
	unset out
	exec mv orthologsMGI_RefSeq.txt Intron_Data/orthologsMGI_RefSeq.txt
	puts "List of orthologous genes converted to RefSeqs"
	

	# Summary Information
	puts "Number of orthologous IDs examined:               $orthocount"
	puts "Mouse IDs converted:                              $convertedmouse"
	puts "Mouse IDs not converted:                          $notconvertedmouse"
	puts "Rat IDs converted:                                $convertedrat"
	puts "Rat IDs not converted:                            $notconvertedrat"
	puts "Number of orthologous IDs successfully converted: $refseqpaircount"
	
	# Unset variables
	unset all EntrezList nottheremouse notthererat orthocount convertedmouse notconvertedmouse convertedrat notconvertedrat refseqpaircount
	
	puts "\nVariables still open in INTRONS04_convertgeneid : [info local]"
	
return
}