########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 15/10/2007 (As part of MMBIntrons work)
# Modified on 	: 11/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  11/05/2009 (Removal of output negative/missing counts)
#				  11/05/2009 (Relaxation of tests of true orthology).
# 						     
# Input files 	: GeneData_Mouse.txt (Output by tcl script INTRONSextractintrons.tcl)
#				  GeneData_Rat.txt (Output by tcl script INTRONSextractintrons.tcl)
#				  orthologsMGI_RefSeq.txt (Output by tcl script INTRONSconvertgeneid.tcl)
#
# Output files  : orthologs_TrueRodent.txt
#				  
# This script aims to generate a list of true orthologous genes.
# The script uses the list of orthologous MGI refseqs.
# First it determines whether intronic sequences have been extracted from the UCSC table browser for both genes.
# This loop removes the refseq once found to make searching faster.
# Then it determines whether these genes are true orthologs based on the following tests:
# Data pertaining to each gene is extracted from the two GeneData_Organisms.txt files.
# For each gene this has data on : Refseq, Chromosome, Intron number, Exon number, Exon phases, Exon lengths, Intron lengths.
# This information is stored in arrays, resetting the chromosomal location to control for uknowns and autosomal locations.
# Using this data it is then determined whether the gene has:
#		Same number of exons?
#		Same number of introns?
#		Same phases of exons?
#		Same chromosomal class location? (A, X or Y only)
# If these conditions are met, the orthologous refseq IDs are output to a file
# NB.  MGI and NCBI use provisional genes whereas UCSC is restricted to known genes. Provisional XM_ genes are therefore removed)
# NB.  True orthology could only be determined for autosomal and X-linked genes as full gene sequences do not exist for rat Y-linked genes.
#	   It is therefore not possible to compare number or phase of exons for Y-linked genes.
# 	   The list of orthologs output to 'orthologs_TrueRodent.txt' contains only autosomal and X-linked genes.		
#
########################################################################################################################################################

package provide INTRONS05_identifyorthologs 1.0

proc INTRONS05_identifyorthologs {} {
	
	puts "-----------------------------------------------------------"
	puts "05             IDENTIFYING ORTHOLOGOUS GENES               \n"

	# Make variables global
	global contents
	
	# Create list of all refseqIDs of genes containing introns that are not duplicate gene IDs
	set refseqList {}
	
	
	
	
	
	# EXTRACT INFORMATION ABOUT EACH GENE IN EACH SPECIES & SET IN ARRAYS
	# THIS DATA WILL BE USED TO DETERMINE IF GENES ARE TRUE ORTHOLOGS BASED ON EXON AND INTRON NUMBER, PHASE, LENGTH ETC.
	puts "Extracting gene data for use determining true orthology"
	
	cd Intron_Data
	set GeneDataFiles [glob GeneData*.txt]
	foreach GeneDataFile $GeneDataFiles {
	
	    # Identify organism
	    regexp {GeneData_(.+)\.txt} $GeneDataFile all organism
	    
	    # Extract contents from file
	    openfile $GeneDataFile
	    set contents [split $contents \n]
	    set contents [lrange $contents 1 end-1]
	    
	    # Separate out information and reset as arrays for each type of information
	    foreach datarow $contents {
		
			# Extract information
			regexp {(.+):(.+):(.+):(.+):(.+):(.+):(.+)} $datarow all refseqID chromosome numbintrons numbexons exonphases exonlengths intronlengths
			
			# Save list of all refseqs extracted from UCSC file
			lappend refseqList $refseqID
		
			# Reset chromosomal class accounting for unknown, random and autosomal locations
			regexp {chr(.+)} $chromosome all location
			if {[regexp {[0-9]} $location all] & ![regexp {[^0-9]} $location all]} {
		    	set chromosomeclass A
			} elseif {[regexp {[xX]} $location all] & ![regexp {[^xX]} $location all]} {
		    	set chromosomeclass X
			} elseif {[regexp {[yY]} $location all] & ![regexp {[^yY]} $location all]} {
		    	set chromosomeclass Y
			} else {
		    	set chromosomeclass U
			}
				
			# Set gene characteristics as arrays for ortholog checking
			set CHROMOSOME($organism,$refseqID) $chromosomeclass
			set NUMBERINTRONS($organism,$refseqID) $numbintrons
			set NUMBEREXONS($organism,$refseqID) $numbexons
			set EXONPHASES($organism,$refseqID) $exonphases
			set EXONLENGTHS($organism,$refseqID) $exonlengths
			set INTRONLENGTHS($organism,$refseqID) $intronlengths
		
			# Unset variables
			unset all refseqID chromosome numbintrons numbexons exonphases exonlengths intronlengths location chromosomeclass	    
	    }
		unset datarow contents
	    
	    # Unset variables
	    unset organism
	}
	unset GeneDataFile GeneDataFiles
	cd ..

	puts "Gene Data extracted for use determining true orthology"
	
	
	
	
	
	
	
	# Extract contents of orthologous refseqID file
	cd Intron_Data
	openfile orthologsMGI_RefSeq.txt
	cd ..
	set contents [split $contents \n]
	set contents [lreplace $contents end end]
	
	
		
	# Create output file for true ortholog refseqIDs	
	set out [open orthologs_TrueRodent.txt w]
	puts $out "Mouse_RefSeqID\tRat_RefSeqID"
	
		
	
	# Initiate counts for program information and checking
	set countFoundMouse 0
	set countFoundRat 0
	set countFoundBoth 0

	
	# FIND MGI ORTHOLOGS AND CHECK THEY ARE TRUE ORTHOLOGS 
	
	puts "Determining if data exists for MGI orthologs & testing true orthology"
	
	# Determine whether each pair of orthologous refseqIDs contain the same numbers and phases of exons
	foreach orthologID $contents {
		
		# ******************
		# FIND MGI ORTHOLOGS
		
	    # Extract mouse and rat orthologous refseqIDs
	    set mouseRefSeq [lindex $orthologID 0]
	    set ratRefSeq [lindex $orthologID 1]
	    
	    # Initiate counts for to determine whether each ortholog identified in dataset
	    set FoundMouse 0
	    set FoundRat 0
	    
		# *************************************************************************
		# WERE SEQUENCES EXTRACTED FROM UCSC GENOME FILES FOR THESE TWO ORTHOLOGS??
		
	    # Determine if mouse refseqID is present in list of all refseqIDs for which sequences extracted
	    if {[lsearch $refseqList $mouseRefSeq] >= 0} {
			incr FoundMouse
			incr countFoundMouse
			# Remove ID from list to make future searching faster
			set refseqList [lreplace $refseqList [lsearch $refseqList $mouseRefSeq] [lsearch $refseqList $mouseRefSeq]]
	    }
	    
	    # Determine if rat refseqID is present in list of all refseqIDs for which sequences extracted
	    if {[lsearch $refseqList $ratRefSeq] >= 0} {
			incr FoundRat
			incr countFoundRat
			# Remove ID from list to make future searching faster
			set refseqList [lreplace $refseqList [lsearch $refseqList $ratRefSeq] [lsearch $refseqList $ratRefSeq]]
	    }
	
	
		# ****************************************
		# IF SEQUENCES EXIST FOR MGI ORTHOLOGS....
		# .... ARE THEY TRUE ORTHOLOGS??	
	
	    # If both mouse and rat orthologs are found in dataset
	    if {[expr $FoundMouse + $FoundRat] == 2} {
			incr countFoundBoth
				
			# Initiate error counts to enable identification of true orthologs
			set errorTotal 0
						
			# USE EXON CHARACTERISTICS TO IDENTIFY TRUE ORTHOLOGUES 		
						# Same number of exons?
						# Same exon phases?
						# Same chromosome type?
						# Same number of introns?
				
			# NUMBER OF EXONS
			if {"$NUMBEREXONS(Mouse,$mouseRefSeq)" != "$NUMBEREXONS(Rat,$ratRefSeq)"} {incr errorTotal}
	
			# EXON PHASES
			if {"$EXONPHASES(Mouse,$mouseRefSeq)" != "$EXONPHASES(Rat,$ratRefSeq)"} {incr errorTotal}
	
			# NUMBER INTRONS
			if {"$NUMBERINTRONS(Mouse,$mouseRefSeq)" != "$NUMBERINTRONS(Rat,$ratRefSeq)"} {incr errorTotal}
			
			# CHROMOSOMAL CLASS
			if {"$CHROMOSOME(Mouse,$mouseRefSeq)" != "$CHROMOSOME(Rat,$ratRefSeq)"} {incr errorTotal}
			
			
			# If gene passes tests, send to orthologs to output file
			if {$errorTotal == 0} {
				puts $out "$mouseRefSeq\t$ratRefSeq"
			}
			unset errorTotal
		
		}
		
	    # Unset variables
	    unset mouseRefSeq ratRefSeq FoundMouse FoundRat
	
	}
	unset orthologID contents
	
	puts "All true orthologs with sequences extracted"
	
	# Close ortholog output file and move to Intron_Data directory
	close $out
	unset out
	exec mv orthologs_TrueRodent.txt Intron_Data/orthologs_TrueRodent.txt
	
	# Summary Information
	puts "Data exists for mouse ortholog ID\t\t: $countFoundMouse"
	puts "Data exists for rat ortholog ID\t\t\t: $countFoundRat"
	puts "Found Both refseqIDs\t\t\t\t: $countFoundBoth"
	

	# Unset variables
	unset CHROMOSOME NUMBERINTRONS NUMBEREXONS EXONPHASES EXONLENGTHS INTRONLENGTHS 
	unset refseqList countFoundMouse countFoundRat countFoundBoth   

	puts "\nVariables still open in INTRONS05_identifyortholog : [info local]"
	
return
}