########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 26/11/2007 (As part of MMBIntrons work)
# Modified on 	: 11/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				   						     
# Input files 	: orthologs_TrueRodent.txt
#				  IN_$mouseRefSeq.txt (output by tcl script INTRONSextractintrons.tcl and stored in INTRONS_Mouse directory)
#				  IN_$ratRefSeq.txt (output by tcl script INTRONSextractintrons.tcl and stored in INTRONS_Rat directory)
#
# Output files  : Orth-$orthologcount-chr-$chromosome directory for each pair of orthologous genes stored in ORTHOLOGS directory
# 				  rn-$ratRefSeq-$Rintronnumber.txt (Orthologous mouse introns stored in Orth... directory)
# 				  mm-$mouseRefSeq-$Mintronnumber.txt (Orthologous rat introns stored in Orth... directory)
# 				  orthologs_index.txt (Contains a list of orthologous refseq ids for each ortholog ID number)
# 
# AIM: To separate each intronic sequence into a separate FASTA file named according to species (mm or rn), RefSeqID and intron number
# NB: Intron numbering derived from a count initiated at 1 where introns are extracted in order and the intron count is increased for each intron
# MODIFIED 31/1/08: Intron numbering derived from region number in original UCSC input files, controling for whether there is a first exon as this will determine subsequent intron numbering
#
# Create directory (ORTHOLOGUES) to store orthologs intron sequence files in
# Extract list of orthologous RefSeq gene IDs from orthologs_TrueRodent.txt and go through each pair of orthologous gene IDs in turn, identifying the mouse and rat refseq IDs
# From the INTRONS directory for the each organism, identify the file containing the required intron sequences (Files named by refseqID IN_$organisimRefSeq.txt)
# Create a directory Orth-$orthologcount-chr-$chromosome to save orthologs intron files in (Chromosome numbering different in mouse and rat so use mouse chromosomal location as the reference location for the orthologous pair)
# For each organism, send each intron to a separate file (mouse: mm-$mouseRefSeq-$Mintroncount.txt and rat: rn-$ratRefSeq-$Rintroncount.txt), then move intron file to ortholog directory Orth-$orthologcount-chr-$chromosome
# Move ortholog directory Orth-$orthologcount-chr-$chromosome to main ORTHOLOGUES directory
#
########################################################################################################################################################

package provide INTRONS06_extractorthologs 1.0

proc INTRONS06_extractorthologs {} {

	puts "-----------------------------------------------------------"
	puts "06    PAIR ORTHOLOGOUS INTRON SEQUENCES FOR ALIGNMENT      \n"

    # Make variables global
    global contents
    
	# Move to Intron_Data directory and do all work in this directory
	cd Intron_Data
	
    # Create a directory to store each directory containing orthologous intron files
    if {![file exists ORTHOLOGS]} {
		puts "No ORTHOLOGS directory was found"
		exec mkdir ORTHOLOGS
		puts "New ORTHOLOGS directory created"
    } elseif {[file exists ORTHOLOGS]} {
		exec rm -r ORTHOLOGS
		puts "ORTHOLOGS directory was found and removed"
		exec mkdir ORTHOLOGS
		puts "New ORTHOLOGS directory created"
    }

	# Creat output file for index of gene IDs for each ortholog number
	set outIndex [open orthologs_index.txt w]
	puts $outIndex "Ortholog_Number\tClass\tMouse_RefSeqID\tRat_RefSeqID"

	# Initiate count for each orthologue pair
	set orthologcount 0 

    # Initiate counts for the number of genes and introns delt with
    set countedgenes 0
    set countedintrons 0
    
    # For each pair of orthologs refseqIDs, find the corresponding intron files and reformat in preparation for alignment
	openfile orthologs_TrueRodent.txt
    set orthologlist [split $contents \n]
    set orthologlist [lrange $orthologlist 1 end-1]
    unset contents
    
	foreach ortholog $orthologlist {
		# Increase count of orthologs
		incr orthologcount
		
		# Increase count of total genes dealt with
		incr countedgenes
		
		# Identify orthologs mouse and rat refseqIDs
		set mouseRefSeq [lindex $ortholog 0]
		set ratRefSeq [lindex $ortholog 1]
		
		# ******************
		# FIND MOUSE INTRONS
		
		# Find file containing mouse introns and reset as a sublist
		cd INTRONS_Mouse
		openfile IN_$mouseRefSeq.txt
		set intronsMouse [split $contents >]
		set intronsMouse [lreplace $intronsMouse 0 0]
		unset contents
		cd ..
		
		# Determine the chromosomal location of the mouse ortholog as a reference for future chromosomal classification
		set findchromosome [lindex $intronsMouse 0]
		set findchromosome [split $findchromosome \n]
		set findchromosome [lindex $findchromosome 0]
		regexp {.+chr(.+):.+} $findchromosome all chromosome
		unset findchromosome all
		
		# Create directory to save orthologs intron files in
		exec mkdir Orth-$orthologcount-chr-$chromosome	
		
		# Inititate variable to compare intron numbers to
		set Mpreviousintron 0
		
		# Send each pair of orthologous introns to a separate file
		foreach Mintron $intronsMouse {
			
	    	# Determine the intron number for intron file naming from the annotated region
			set annotation [split $Mintron \n]
			set annotation [lindex $annotation 0]
			
			# Determine inton  number
			regexp {mm:NM_[0-9]+_(.+):chr[\w]+} $annotation all regionnumber
			# There is no firt exon so region 0 contains the first intron
			if {$regionnumber == 0} {
				set Mintronnumber [expr {[expr {$regionnumber + 2}] / 2}]
				if {[expr {$Mintronnumber + 1}] == $Mpreviousintron} {
					puts "ERROR... Current intron number $Mintronnumber not 1 greater than previous intron number $Mpreviousintron"
					exit
				} else {			
					set Mpreviousintron $Mintronnumber
				}
			# There is no first exon so regions containing introns are even numbers
			} elseif {[expr $regionnumber % 2] == 0} {
				set Mintronnumber [expr {[expr {$regionnumber + 2}] / 2}]
				if {[expr {$Mintronnumber + 1}] == $Mpreviousintron} {
					puts "ERROR... Current intron number $Mintronnumber not 1 greater than previous intron number $Mpreviousintron"
					exit
				} else {			
					set Mpreviousintron $Mintronnumber
				}
			# There is a first exon so regions containing introns are odd numbers
			} else {
				set Mintronnumber [expr {[expr {$regionnumber + 1}] / 2}]
				if {[expr {$Mintronnumber + 1}] == $Mpreviousintron} {
					puts "ERROR... Current intron number $Mintronnumber not 1 greater than previous intron number $Mpreviousintron"
					exit
				} else {			
					set Mpreviousintron $Mintronnumber
				}
			}	

			# Create intron output file referenced by species, refseqID and intron number, then send intron to output file in FASTA format
			set out [open mm-$mouseRefSeq-$Mintronnumber.txt w]
	    	puts $out ">$Mintron"
	    	close $out
	    	unset out
	    	# Move intron file to ortholog directory
	    	exec mv mm-$mouseRefSeq-$Mintronnumber.txt Orth-$orthologcount-chr-$chromosome/mm-$mouseRefSeq-$Mintronnumber.txt
			# Unset variables
			unset Mintronnumber annotation regionnumber
		}
		# Unset variable
		unset Mintron intronsMouse

		# ****************
		# FIND RAT INTRONS


		# Find file containing rat introns and reset as a sublist
		cd INTRONS_Rat
		openfile IN_$ratRefSeq.txt
		set intronsRat [split $contents >]
		set intronsRat [lreplace $intronsRat 0 0]
		unset contents
		cd ..
		
		
		# Inititate variable to compare intron numbers to
		set Rpreviousintron 0
		
		# Send each pair of orthologous introns to a separate file
		foreach Rintron $intronsRat {
			# Determine the intron number for intron file naming from the annotated region
			set annotation [split $Rintron \n]
			set annotation [lindex $annotation 0]
			
			# Determine inton  number
			regexp {rn:NM_[0-9]+_(.+):chr[\w]+} $annotation all regionnumber
			# There is no firt exon so region 0 contains the first intron
			if {$regionnumber == 0} {
				set Rintronnumber [expr {[expr {$regionnumber + 2}] / 2}]
				if {[expr {$Rintronnumber + 1}] == $Rpreviousintron} {
					puts "ERROR... Current intron number $Rintronnumber not 1 greater than previous intron number $Rpreviousintron"
					exit
				} else {
					set Rpreviousintron $Rintronnumber
				}
			# There is no first exon so regions containing introns are even numbers
			} elseif {[expr $regionnumber % 2] == 0} {
				set Rintronnumber [expr {[expr {$regionnumber + 2}] / 2}]
				if {[expr {$Rintronnumber + 1}] == $Rpreviousintron} {
					puts "ERROR... Current intron number $Rintronnumber not 1 greater than previous intron number $Rpreviousintron"
					exit
				} else {
					set Rpreviousintron $Rintronnumber
				}
			# There is a first exon so regions containing introns are odd numbers
			} else {
				set Rintronnumber [expr {[expr {$regionnumber + 1}] / 2}]
				if {[expr {$Rintronnumber + 1}] == $Rpreviousintron} {
					puts "ERROR... Current intron number $Rintronnumber not 1 greater than previous intron number $Rpreviousintron"
					exit
				} else {			
					set Rpreviousintron $Rintronnumber
				}
			}

	    	# Increase count of total introns dealt with
	    	incr countedintrons
	    	# Send intron annotation and sequence to a unique file
	    	set out [open rn-$ratRefSeq-$Rintronnumber.txt w]
	    	puts $out ">$Rintron"
	    	close $out
	    	unset out
	    	# Move intron file to ortholog directory
	    	exec mv rn-$ratRefSeq-$Rintronnumber.txt Orth-$orthologcount-chr-$chromosome/rn-$ratRefSeq-$Rintronnumber.txt
	    	# Unset variables
	    	unset Rintronnumber annotation regionnumber
		}
		# Unset variables
		unset Rintron intronsRat
		
		# Move ortholog file directory to main ortholog directory
		exec mv Orth-$orthologcount-chr-$chromosome ORTHOLOGS/Orth-$orthologcount-chr-$chromosome	
		
		
		# Send gene IDs to ortholog index output file
		if {$chromosome == "X"} {set class X} else {set class A}
		puts $outIndex "$orthologcount\t$class\t$mouseRefSeq\t$ratRefSeq"
		
		# Unset variables
		unset mouseRefSeq ratRefSeq chromosome class
    }
	unset ortholog orthologlist


	# Close ortholog index output file
	close $outIndex
	unset outIndex
	
	# Return from Intron_Data directory to main directory
   	cd ..


    # Summary Information
    puts "Orthologs with introns prepared for alignment:\t$orthologcount"
    puts "Number of orthologous genes extracted:\t\t\t$countedgenes"
    puts "Number of orthologous introns extracted:\t\t$countedintrons"
    
    # Unset variables
    unset countedintrons countedgenes
	unset orthologcount
    
    puts "\nVariables still open in INTRONS06_extractorthologs : [info local]"
    
return
}
