########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 23/11/2007 (As part of MMBIntrons work)
# Modified on 	: 12/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  12/05/2009 (Ortholog number determined from ortholog_index.txt) 
#				  12/05/2009 (Checks on whether files already exist)
#				  12/05/2009 (Addition of Y-linked data to orthologs_index.txt file)
#				  12/05/2009 (Addition of Y-linked intron positions to ouput files)
#				  17/08/2009 (NM_ added to rat zfy annotation)
#   
# Input files 	: orthologs_index.txt (output from tcl script INTRONS06_extractorthologs.tcl)
#				  mmu_rno_introns_Y.tab (Created by Nizar and obtained from LDH)
#				  Zfy_int_aln_ed.tfa (Created by Nizar and obtained from LDH)
#				  
# Output files  : Orth-$orthologcount-chr-$chromosome directory for each pair of orthologous genes stored in existing ORTHOLOGS directory
# 				  rn-$ratRefSeq-$Rintronnumber.txt (Orthologous mouse introns stored in Orth... directory)
# 				  mm-$mouseRefSeq-$Mintronnumber.txt (Orthologous rat introns stored in Orth... directory)
# 				  orthologs_index.txt (Contains a list of orthologous refseq ids for each ortholog ID number)
# 
# For Y-linked JARID, EIF and UBE genes, extract intron sequences and annotations (including gene name) from the input mmu_rno_introns_Y.tab file
# For each new gene, create a new directory Orth-$orthologcount-chr-Y for the ortholog
# For each pair of orthologous introns, send intron sequences and formatted annotation lines to separate files (mouse: mm-$MouseYGene-$introncount.txt & rat: rn-$RatYGene-$introncount.txt)
# Move all introns per orthologous genes to ortholog directory, then move ortholog directory to main ORTHOLOGS directory ready for alignment
# For Y-linked ZFY genes (single introns available), open zfy file (Zfy_int_aln_ed.tfa) and extract the mouse and rat sequences, removing all alignment indels and spaces (To control for variation in alignment parameters, gene will be re-aligned with the rest of the data set)
# Set annotation information for each gene based on format used for other orthologs
# Send each intron to output files (mouse: mm-NM_009570-0.txt) and (rat: rn-NM_zfy-0.txt)
# Move both intron files to new Orth-$orthologcount-chr-Y directory, then move this directory to main ORTHOLOGS directory ready for alignment
# The ortholog count of each Y-linked gene is determined from the the list of all autosomal and X-linked orthologs.
# Any exisiting Y-linked details are removed from the ortholog_index.txt file (from previous runs of the script) and the count is set as the maximum count + 1
# The Y-trimmed contents are then returned to the output file and new details about Y-linked orthologous genes are added to the ortholog_index.txt file. 
#
########################################################################################################################################################

package provide INTRONS07_ratYintrons 1.0

proc INTRONS07_ratYintrons {} {
	
	puts "-----------------------------------------------------------"
	puts "07   EXTRACT UNALIGNED Y-LINKED SEQUENCES FOR ALIGNMENT    \n"
	
	# Make variables global
    global contents
    	
	# Move to Intron_Data directory and do all work in this directory
	cd Intron_Data




	# Open output file for index of gene IDs for each ortholog number
	openfile orthologs_index.txt
	
	set contents $contents
	set contents [split $contents \n]
	set headings [lindex $contents 0]
	set contents [lrange $contents 1 end-1]
	
	# Remove Y-linked details
	while {[lindex [lindex $contents end] 1] == "Y"} {
		set contents [lreplace $contents end end]
	} 
	
	# Set ortholog count of last X-linked gene
	set orthologcount [lindex [lindex $contents end] 0]
	puts "Ortholog count reset as $orthologcount (last autosomal and X-linked)"
	
	# Send Y-trimmed results back to orthologs_index.txt file and leave open ready to add new Y-linked data
	set out [open orthologs_index.txt w]
	regsub -all {\{} $contents "" contents
	regsub -all {\}\s} $contents "\n" contents
	regsub -all {\}} $contents "" contents
	puts $out "$headings\n$contents"
	unset contents headings 
	puts "Original orthologs_index.txt contents returned to file ready to add Y-linked data"
	
	
	
	
	
	
	# ******************************************************************************************************
    # DEAL WITH Y-LINKED JARID/EIF/UBE GENES
	puts "Extracting rat Y sequences from file mmu_rno_introns_Y.tab"
    
    # Extract contents of rodent Y-linked orthologous sequences file and separate into orthologous introns
	cd ../Input_Files
    openfile mmu_rno_introns_Y.tab
	cd ../Intron_Data
    set introns [split $contents \n]
    set introns [lreplace $introns end end]
    unset contents
        
    # For each pair of orthologous introns, reformat data in preparation for alignment and move to ORTHOLOGS directory
    set previousGene NoneYet
    foreach intron $introns {
	
		set AnnotationMouse [lindex $intron 0]
		set AnnotationRat [lindex $intron 1]
		set SequenceMouse [lindex $intron 2]
		set SequenceRat [lindex $intron 3]
	
		# Identify Y-linked gene name from annotation
		regexp {.+refGene_(NM_.+)_([0-9]+);chrY:([0-9]+)-([0-9]+)} $AnnotationMouse all MouseYGene MouseYRegion intronstart intronend
		regexp {.+refGene_(.+)_[0-9]+} $AnnotationRat all RatYGene

		# Determine intron number from mouse region
		set intronnumber [expr {[expr {$MouseYRegion + 1}] / 2}]

		# If the current gene is different to the previous gene, create a new directory for the ortholog
		if {$RatYGene != $previousGene} {
			# Increase ortholog count ready for the next gene
	    	incr orthologcount

			# Send gene IDs to ortholog index output file
			puts $out  "$orthologcount\tY\t$MouseYGene\t$RatYGene"			
	    	# If an ortholog directory has already been created, move it to main ORTHOLOGS directory
	    	if {$previousGene != "NoneYet"} {
				cd ORTHOLOGS
				if {[file exists $directoryname]} {exec rm -r $directoryname}
				cd ..
				exec mv $directoryname ORTHOLOGS/$directoryname
   			}
	    
			# Reset orthGene and directory name for the current gene
	    	set previousGene $RatYGene
	    	set directoryname Orth-$orthologcount-chr-Y
	
	    	# Create a directory for the orthologs
			if {[file exists $directoryname]} {exec rm -r $directoryname}
	    	exec mkdir $directoryname
	    	puts "Directory $directoryname created"
	    	# Create output files with correct filenames for intronic sequences in preparation for alignment
			set intronoutMouse [open mm-$MouseYGene-$intronnumber.txt w]
			set intronoutRat [open rn-$RatYGene-$intronnumber.txt   w]
	    	# Send annotation and sequence to relevant file
	    	puts $intronoutMouse ">mm:$MouseYGene:chrY:$intronstart-$intronend\n$SequenceMouse"
	    	puts $intronoutRat ">rn:$RatYGene:chrY\n$SequenceRat"
	    	# Close output files and move to newly created directory
	    	close $intronoutMouse
			exec mv mm-$MouseYGene-$intronnumber.txt  $directoryname/mm-$MouseYGene-$intronnumber.txt
	    	close $intronoutRat
	    	exec mv rn-$RatYGene-$intronnumber.txt  $directoryname/rn-$RatYGene-$intronnumber.txt
	    	unset intronoutMouse intronoutRat

		# If the current gene is the same as the previous gene, send the output files to the existing directory
		} elseif {$RatYGene == $previousGene} {

	    	# Create output files with correct filenames for intronic sequences in preparation for alignment
			set intronoutMouse [open mm-$MouseYGene-$intronnumber.txt w]
			set intronoutRat [open rn-$RatYGene-$intronnumber.txt   w]
	    	# Send annotation and sequence to relevant file
	    	puts $intronoutMouse ">mm:$MouseYGene:chrY:$intronstart-$intronend\n$SequenceMouse"
	    	puts $intronoutRat ">rn:$RatYGene:chrY\n$SequenceRat"
	    	# Close output files and move to newly created directory
	    	close $intronoutMouse
			exec mv mm-$MouseYGene-$intronnumber.txt  $directoryname/mm-$MouseYGene-$intronnumber.txt
	    	close $intronoutRat
			exec mv rn-$RatYGene-$intronnumber.txt  $directoryname/rn-$RatYGene-$intronnumber.txt
  			unset intronoutMouse intronoutRat
		}
		
		
		# Unset variables
		unset AnnotationMouse AnnotationRat
		unset MouseYGene MouseYRegion RatYGene intronstart intronend
		unset SequenceMouse SequenceRat intronnumber
    }

    # Send final directory to main ORTHOLOGS directory
	cd ORTHOLOGS
	if {[file exists $directoryname]} {exec rm -r $directoryname}
	cd ..
    exec mv $directoryname ORTHOLOGS/$directoryname
    
	# Unset variables
    unset all intron introns previousGene directoryname 


 	# ******************************************************************************************************
    # DEAL WITH Y-LINKED ZFY GENES
	puts "Extracting Y-linked zfy genes from file Zfy_int_aln_ed.tfa" 
    
	# Increase ortholog count ready for the next gene
	incr orthologcount
	
	# Send gene IDs to ortholog index output file
	puts $out  "$orthologcount\tY\tNM_009570\tNM_zfy"
    
	# Open zfy file and extract gene information
	cd ../Input_Files
    openfile Zfy_int_aln_ed.tfa
	cd ../Intron_Data
    set contents [split $contents >]
    set mouse [lindex $contents 2]
    set rat [lindex $contents 1]
    unset contents
        
    # Extract the mouse sequence and remove alignment indels and spaces
    regexp {.+repeatMasking=none(.+)} $mouse all mousesequence
    unset all mouse
    regsub -all {\s} $mousesequence "" mousesequence
    regsub -all {\-} $mousesequence "" mousesequence
    
    # Extract the rat sequence and remove alignment indels and spaces
    regexp {.+intron\)(.+)} $rat all ratsequence
    unset all rat
    regsub -all {\s} $ratsequence "" ratsequence
    regsub -all {\-} $ratsequence "" ratsequence

    # Set annotation information for each gene based on format used for other orthologs (Set mannually)  
    set mouseannotation ">mm:NM_009570_11:chrY:63039-63951"
    set ratannotation ">rn:NM_zfy_11:chrY"
    
    # Create output files for each intron in preparation for lagan alignment - Sequence is last intron allocate intron number >0 to prevent elimination from analysis 
    set outmouse [open mm-NM_009570-6.txt w]
    puts $outmouse "$mouseannotation\n$mousesequence"
    close $outmouse
    unset outmouse mouseannotation mousesequence
    
    set outrat [open rn-NM_zfy-6.txt w]
    puts $outrat "$ratannotation\n$ratsequence"
    close $outrat
    unset outrat ratannotation ratsequence

    # Create ortholog directory for zfy intron files and move to main orthologs directory
	if {[file exists Orth-$orthologcount-chr-Y]} {exec rm -r Orth-$orthologcount-chr-Y}
	exec mkdir Orth-$orthologcount-chr-Y
	puts "Directory Orth-$orthologcount-chr-Y created"
    exec mv mm-NM_009570-6.txt Orth-$orthologcount-chr-Y/mm-NM_009570-6.txt
    exec mv rn-NM_zfy-6.txt Orth-$orthologcount-chr-Y/rn-NM_zfy-6.txt
	cd ORTHOLOGS
	if {[file exists Orth-$orthologcount-chr-Y]} {exec rm -r Orth-$orthologcount-chr-Y}
	cd ..
    exec mv Orth-$orthologcount-chr-Y ORTHOLOGS/Orth-$orthologcount-chr-Y

	
	# Close ortholog index output file
	close $out
	unset out
    	
	# Unset variables
	unset orthologcount
	
	# Return from Intron_Data directory to main directory
   	cd ..
	
    puts "\nVariables open in INTRONS07_ratYintrons : [info local]"

return
}
