########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 11/12/2007
# Modified on 	: 13/05/2009 (Change script structure to fit in with current analysis)
#
# Input files 	: Procedure receives 2 INDEL FREE alignments when the procedure is called.				  
#				  
# Output files  : Procedure hands back an answer to the test.
#				  This is stored in the variable 'runstest' and is either 'passed' or 'failed'
# 
# 
# Input : Aligned mouse and rat sequences passed from alignmentparameters subscript
# Ensure that both alignments are the same length (correctly extracted in alignment parameters). If not, exit the program
#
#
# _______________________________________________________________________________________________________
# String index: 						0    1    2    3    4    5    6    7    8    9     10    11
# Previous position:					0    1    2    3    4    5    6    7    8    9     10    11
# Current position:						0+1  1+1  2+1  3+1  4+1  5+1  6+1  7+1  8+1  9+1   10+1  11+1  
# 
# Mouse Alignment:						A    T    G    A    C    G    C    G    G    T     A     A
# Rat Alignment:						A    T    G    G    C    A    T    G    G    T     A     G			
#															
# Diverged or conserved?:				0    0    0    1    1    1    1    0    0    0     0     1 
#												    \/					\/                   \/
# Switch?:								            S                    S                    S     
#________________________________________________________________________________________________________
# SWITCH = {Previous Mouse != Previous Rat (1) } -> {Current Mouse == Current Rat (0) }
# SWITCH = {Previous Mouse == Previous Rat (0) } -> {Current Mouse != Current Rat (1) }
#________________________________________________________________________________________________________
#
#
#
# To make runs test faster, only pass through the aligned sequences once, determining whether bases are conserved or diverged and whether there is a switch in state in one run
# Although string index starts at 0, postion starts at 1 so that there is a comparison between that and the previous postition to determine whether there is a switch
# Identify first position in each alignment (Index position 1 - 1 = 0) (alignment 1 = mouse, alignment 2 = rat)
# I the first position of either sequence is an indel "-", move on to the next position by increasing the position count (eliminate "-", only deal with aligned nucleotides)
#
# 28th April 2008 : Modified linear model used to determine predicted number of switches per base based on sequence length and extent of divergence to account for corrected simulation
#
########################################################################################################################################################

package provide INTRONS13_runstest 1.0

proc INTRONS13_runstest {alignment1 alignment2} {
	
	# Make output variable global
	global runstest

	global COUNTalignednucleotides
	global COUNTconserved
	global COUNTdiverged
	global COUNTswitches
	global percentagedivergence
	global Expected
	global Observed

	# Check alignments the same length
	if {[string length $alignment1] != [string length $alignment2]} {
		puts "Error...Alignments for runstestcomparison not the same length"
		exit
	}

	# Initiate counts
	set COUNTalignednucleotides [string length $alignment1]
	set COUNTconserved 0
	set COUNTdiverged 0
	set COUNTswitches 0

	# Determine if diverged or conserved AND if switch in state compared to previous position in one run along the sequence 
	# (quicker than creating a diverged vs conserved sequence and then going through again to in one runethen swithes in state)
	for {set indexposition 0} {$indexposition < [expr {[string length $alignment1] - 1}]} {incr indexposition} {
		
		# Extract bases at FIRST index position
		set pos1seq1 [string index $alignment1 $indexposition]
		set pos1seq2 [string index $alignment2 $indexposition]
		
		# Extract bases at SECOND index position
		set pos2seq1 [string index $alignment1 [expr {$indexposition + 1}]]
		set pos2seq2 [string index $alignment2 [expr {$indexposition + 1}]]
		
		# Ensure none of the positions contain an indel 
		if {$pos1seq1 == "-" | $pos1seq2 == "-" | $pos2seq1 == "-" | $pos2seq2 == "-"} {
			puts "ERROR...Positions contain an indel"
			exit
		}

		# Increase count of conserved or diverged bases for first pervious position (all subsequent counts use current position)
		if {$indexposition == 0} {
			if {$pos1seq1 == $pos1seq2} {
				incr COUNTconserved
			} else {
				incr COUNTdiverged
			}
		}


		# Set PREVIOUS and CURRENT bases
		set previous1 $pos1seq1
		set previous2 $pos1seq2

		set current1 $pos2seq1
		set current2 $pos2seq2
		
		# DETERMINE IF SWITCH IN STATE
		
		# PREVIOUS character state : Conserved (0) or Diverged (1)
		if {$previous1 == $previous2} {
			set STATEprevious 0
		} else {
			set STATEprevious 1
		}
		
		# CURRENT character state : Conserved (0) or Diverged (1)
		if {$current1 == $current2} {
			set STATEcurrent 0
			incr COUNTconserved
		} else {
			set STATEcurrent 1
			incr COUNTdiverged
		}
			
		# SWITCH in character state?
		if {$STATEprevious == $STATEcurrent} {
			set SWITCH NO
		} else {
			set SWITCH YES
			incr COUNTswitches
		}
			
		# Unset variables
		unset pos1seq1 pos1seq2 pos2seq1 pos2seq2 
		unset previous1 previous2 current1 current2
		unset STATEprevious STATEcurrent
		unset SWITCH

	}
	unset indexposition




	############################################################################################################
	# RUNS TEST CALCULATION - BASED ON NUMBER OF SWITCHES PER BASE!!!!!
	############################################################################################################


	# Determine whether sequence is less than z (outlier below 95% CI of expected number of switches in state    
	# Calculate Expected (Z) according to combined model for 95% CI switches per base 
	
	# Reset number of diverged bases as a floating point and calculate % divergence (no. diverged bases / number of aligned bases * 100)
	set COUNTdiverged [format %f $COUNTdiverged]

	set percentagedivergence [expr {[expr {$COUNTdiverged / $COUNTalignednucleotides}] * 100.0}]

	# Calculate expected number of switches per base based on randomised RUNS TEST model linear fit equation
	set Expected [expr {-0.005757 + [expr {$COUNTalignednucleotides * 0.00000026}] + [expr {$percentagedivergence * 0.0192327}] + [expr {$percentagedivergence * $percentagedivergence * -0.000192}] + [expr {0.0000000136 * [expr {[expr {$COUNTalignednucleotides - 20350}] * [expr {$percentagedivergence - 50}]}]}] + [expr {-0.00000000014 * [expr {[expr {$COUNTalignednucleotides - 20350}] * [expr {$percentagedivergence * $percentagedivergence - 3166.67}]}]}]}]

	# Calculate the observed number of switches in state per base
	set COUNTswitches [format %f $COUNTswitches]

	set Observed [expr {$COUNTswitches / $COUNTalignednucleotides}]	
		
	# COMPARISON OF OBSERVED TO EXPECTED

	# If more switches in state than expected, pass the runs test	
	if {$Observed > $Expected} {
		set runstest passed
	# If less switches in state than expected, fail the runs test
	} else {
		set runstest failed
	}		

return
}










