########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 12/12/2007 (As part of MMBIntrons work)
# Modified on 	: 14/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  14/05/2009 (Removal of output statistics and trimming out details printed to screen)
#				  28/05/2009 (Use of CDS positions as genic positions extracted from file CDSpositions-$organism.txt)
#				  17/09/2009 (regexp for Y linked genes can now accept _*1* if present for Y linked (zfy) rat genes)
#
# Modificaion aims :
# 				  Concatenate sequences by gene, and chromosome but also leave introns separate.
# 				  Prepare for running Knc.pl script
# 			
#
# Input files 	: passed_Orth-$ortholog-Intron-$intron-Chr-$chromosome.nuc files in ALIGNMENTS_final directory
# 				  (Files output by tcl script INTRONS09_alignmentparameters.tcl)
#				  CDSpositions-Mouse.txt
#				  CDSpositions_Rat.txt
#				  (Files output by tcl script INTRONS02_extractintrons.tcl)
#				  
# Output files  : Concat_Orth-$ortholog-Chr-$chromosome-Length-$length.nuc in CONCATENATE_genes directory
#				  
# A list of all ortholog IDS was extracted from file names within the ALIGNMENTS_final directory.
# For each orthologID (ie. a gene), all alignment files relating to this gene were indentified.
# Rat and mouse sequences were extracted from each file and appended to a concatenated string for the entire gene.
# Details for the gene refseq and chromosome were extracted from the annotations.
# The maximum and minimum positions across all introns were extracted as the intronic limits of the gene.
# Details of introns per gene were added to the output annotations.
# Concatenated sequences for a single gene were output to file and moved to the new CONCATENATE_genes directory.
# 
# NB: First introns and indels were already removed as part of tcl script INTRONS09_alignmentparameters.tcl
#
########################################################################################################################################################

package provide INTRONS14_concatenategenes 1.0

proc INTRONS14_concatenategenes {} {

	puts "-----------------------------------------------------------"
	puts "14            CONCATENATING ALIGNMENTS BY GENE             \n"

	# Make variables global
	global contents	
		
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data
		
	# Create directory to store all final alignments in
    if {![file exists CONCATENATE_genes]} {
		puts "No CONCATENATE_genes directory was found"
		exec mkdir CONCATENATE_genes
		puts "New CONCATENATE_genes directory created\n"
    } elseif {[file exists CONCATENATE_genes]} {
		exec rm -r CONCATENATE_genes
		puts "CONCATENATE_genes directory was found and removed"
		exec mkdir CONCATENATE_genes
		puts "New CONCATENATE_genes directory created\n"
    }

	# Extract positional limits of coding sequence for each gene for each species
	openfile CDSpositions-Mouse.txt
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		set POSITIONSmouse([lindex $dataline 0]) "[lindex $dataline 1]\t[lindex $dataline 2]"
	}
	unset dataline contents

	openfile CDSpositions-Rat.txt
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		set POSITIONSrat([lindex $dataline 0]) "[lindex $dataline 1]\t[lindex $dataline 2]"
	}
	unset dataline contents

	# Obtain a list of all orthologs with passed alignments available for concatenation
	puts "Finding all orthologs with aligned sequence"
	cd ALIGNMENTS_final
	set passedalignmentfiles [glob passed*.nuc]
	cd ..
	set orthologs {}
	foreach fl $passedalignmentfiles {
		regexp {.+Orth-([0-9]+)-Intron.+\.nuc} $fl all ortholog
		lappend orthologs $ortholog
		unset all ortholog
	}
	unset fl passedalignmentfiles
	set orthologs [lsort -unique -integer $orthologs]
	puts "List of [llength $orthologs] orthologs identified"
	
	# Concatenate by gene
	puts "Concatenating alignments by gene"
	cd ALIGNMENTS_final
	set c 0
	foreach ortholog $orthologs {
		incr c
		puts "$c of [llength $orthologs]"

		set allorthologfiles [glob passed_Orth-$ortholog-Intron*.nuc]

		set mousealignment {}
		set ratalignment {}
		
		set introncount 0
		foreach orthologfile $allorthologfiles {

			# Determine class (X Y or autosomes ie. 1-19) from ortholog file name as annotation regexp's will need to be handled differently for Y-linked genes.
			regexp {passed_.+Chr-([\w]+)\.nuc} $orthologfile all class 
			
			incr introncount

			openfile $orthologfile
			set contents [split $contents \n]
			
			# Append sequences to concatenation for each species
			append mousealignment [lindex $contents 2]
			append ratalignment [lindex $contents 4]

			# Find details for each species
			set mouseannotation [lindex $contents 1]
			set ratannotation [lindex $contents 3]

puts $mouseannotation
puts $ratannotation


			if {$class != "Y"} {
				regexp {.+(mm:NM_[0-9]+)_[0-9]+:(chr.+):.+} $mouseannotation all mouserefseq mousechr
				regexp {.+(rn:NM_[0-9]+)_[0-9]+:(chr.+):.+} $ratannotation all ratrefseq ratchr

				# Determine limits of coding sequence - MOUSE
				regexp {mm:(NM_.+)} $mouserefseq all refseqM
				set mouseSTART [lindex $POSITIONSmouse($refseqM) 0]
				set mouseEND [lindex $POSITIONSmouse($refseqM) 1]
				unset refseqM
			
				# Determine limits of coding sequence - RAT
				regexp {rn:(NM_.+)} $ratrefseq all refseqR
				set ratSTART [lindex $POSITIONSrat($refseqR) 0]
				set ratEND [lindex $POSITIONSrat($refseqR) 1]
				unset refseqR
			
			} else {		
				regexp {.+(mm:NM_[0-9]+)_*[0-9]*:(chr.+):.+} $mouseannotation all mouserefseq mousechr
				regexp {.+(rn:NM_[a-z]+)_*[0-9]*:(chr.+)} $ratannotation all ratrefseq ratchr

				# Determine limits of coding sequence - MOUSE
				regexp {mm:(NM_.+)} $mouserefseq all refseqM
				set mouseSTART [lindex $POSITIONSmouse($refseqM) 0]
				set mouseEND [lindex $POSITIONSmouse($refseqM) 1]
				unset refseqM
			
				# Set limits of coding sequence - RAT
				set ratSTART 0
				set ratEND 0
			
			}
		
			# Unset variables
			unset contents
			unset class
			unset mouseannotation ratannotation			
		}
		unset orthologfile allorthologfiles
		
		# Determine new alignment length		
		set concatenationlength [string length $mousealignment]

		# Send concatenated alignments to output file and move to CONCATENATE_genes directory
		regexp {chr(.+)} $mousechr all chromosome
		set out [open Concat_Orth-$ortholog-Chr-$chromosome-Length-$concatenationlength.nuc w]
		puts $out "\t2\t$concatenationlength\n>$mouserefseq:$mousechr:#Introns-$introncount:$mouseSTART-$mouseEND\n$mousealignment\n>$ratrefseq:$ratchr:#Introns-$introncount:$ratSTART-$ratEND\n$ratalignment"
		close $out
		unset out
		exec mv Concat_Orth-$ortholog-Chr-$chromosome-Length-$concatenationlength.nuc ../CONCATENATE_genes/Concat_Orth-$ortholog-Chr-$chromosome-Length-$concatenationlength.nuc

		# Unset variables
		unset mousealignment ratalignment
		unset mouserefseq mousechr
		unset ratrefseq ratchr
		unset concatenationlength introncount chromosome

	}
	unset ortholog orthologs
	cd ..
	puts "Finished concatenating alignments by gene"
	
	# Return from Intron_Data directory to main directory
   	cd ..

	puts "\nVariables open in INTRONS14_concatenategenes : [info local]"
	
return
}
