########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 12/12/2007 (As part of MMBIntrons work)
# Modified on 	: 14/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  14/05/2009 (Removal of output statistics and trimming out details printed to screen)
#				  14/05/2009 (Complete change of methodology)
#
# Input files 	: passed_Orth-$ortholog-Intron-$intron-Chr-$chromosome.nuc files in ALIGNMENTS_final directory
# 				  (Files output by tcl script INTRONS09_alignmentparameters.tcl)
#				  
# Output files  : Concat_Chr-$chromosome-Length-$length.nuc in CONCATENATE_chromosomes directory
#				  
# A list of all chromosomes was extracted from file names within the ALIGNMENTS_final directory.
# For each chromosome, all alignment files relating to this chromosomes were indentified.
# Rat and mouse sequences were extracted from each file and appended to a concatenated string for the entire chromosome.
# Details of introns per chromosome were added to the output annotations.
# Concatenated sequences for a single chromosome were output to file and moved to the new CONCATENATE_chromosomes directory.
# 
# NB: First introns and indels were already removed as part of tcl script INTRONS09_alignmentparameters.tcl
#
########################################################################################################################################################

package provide INTRONS15_concatenatechromosomes 1.0

proc INTRONS15_concatenatechromosomes {} {
	
	puts "-----------------------------------------------------------"
	puts "15         CONCATENATING ALIGNMENTS BY CHROMOSOME          \n"

	# Make variables global
	global contents	
		
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data
		
	# Create directory to store all final alignments in
    if {![file exists CONCATENATE_chromosomes]} {
		puts "No CONCATENATE_chromosomes directory was found"
		exec mkdir CONCATENATE_chromosomes
		puts "New CONCATENATE_chromosomes directory created\n"
    } elseif {[file exists CONCATENATE_chromosomes]} {
		exec rm -r CONCATENATE_chromosomes
		puts "CONCATENATE_chromosomes directory was found and removed"
		exec mkdir CONCATENATE_chromosomes
		puts "New CONCATENATE_chromosomes directory created\n"
    }

	# Obtain a list of all chromosomes with passed alignments available for concatenation
	puts "Finding all chromosomes with aligned sequence"
	cd ALIGNMENTS_final
	set passedalignmentfiles [glob *.nuc]
	cd ..
	set chromosomes {}
	foreach fl $passedalignmentfiles {
		regexp {.+Chr-(.+)\.nuc} $fl all chromosome
		lappend chromosomes $chromosome
		unset all chromosome
	}
	unset fl passedalignmentfiles
	set chromosomes [lsort -unique $chromosomes]
	puts "List of [llength $chromosomes] chromosomes identified"
	
	# Concatenate by chromosome
	puts "Concatenating alignments by chromosome"
	cd ALIGNMENTS_final
	foreach chromosome $chromosomes {
		
		set allorthologfiles [glob *-Chr-$chromosome.nuc]
		
		set mousealignment {}
		set ratalignment {}
		
		set introncount 0
		foreach orthologfile $allorthologfiles {

			incr introncount
			openfile $orthologfile
			set contents [split $contents \n]

			# Append sequences to concatenation for each species
			append mousealignment [lindex $contents 2]
			append ratalignment [lindex $contents 4]

			unset contents
		}
		unset orthologfile allorthologfiles
		
		# Determine new alignment length		
		set concatenationlength [string length $mousealignment]

		# Send concatenated alignments to output file and move to CONCATENATE_genes directory
		set out [open Concat_Chr-$chromosome-Length-$concatenationlength.nuc w]
		puts $out "\t2\t$concatenationlength\n>Mouse:#Introns-$introncount\n$mousealignment\n>Rat:#Introns-$introncount\n$ratalignment"
		close $out
		unset out
		exec mv Concat_Chr-$chromosome-Length-$concatenationlength.nuc ../CONCATENATE_chromosomes/Concat_Chr-$chromosome-Length-$concatenationlength.nuc

		# Unset variables
		unset mousealignment ratalignment
		unset concatenationlength introncount

	}
	unset chromosome chromosomes
	cd ..
	puts "Finished concatenating alignments by chromosome"
	
	# Return from Intron_Data directory to main directory
  	cd ..

	puts "\nVariables open in INTRONS15_concatenatechromosomes : [info local]"
	
return
}