########################################################################################################################################################
# Written by  	: Catherine J Pink
#
# Created on  	: 28/11/2007 (As part of MMBIntrons work)
# Modified on 	: 14/05/2009 (Change of directory names and script structure to fit in with current analysis)
#				  14/05/2009 (Complete restructuring of script)
#				  05/06/2009 (Removal of genes with -1 error results)		  
#
# Input files 	: All Concat_Chr-$chromosome-Length-$alignmentlength.nuc files in CONCATENATE_chromosomes directory
# 				  (Output by tcl script INTRONS15_concatenatechromosomes.tcl)
#				  
# Output files  : K_noncoding_out.tab (from Knc.pl script - deleted)
#				  K_gene.txt
#				  K_gene.xls
#
# This script calculates intronic substitution rates for chromosomes.
# It moves the Knc.pl script from the mmREPLICATION_tcl.tcl library to the CONCATENATE_chromosomes directory containing the aligned genic sequences.
# The Knc.pl script is executed, generating the output file K_noncoding_out.tab.
# The K_noncoding_out.tab file is moved to the new RESULTS directory.
# The contents of the K_noncoding_out.tab file are reformated to remove file and sequence names.
# Separate chromosome, alignment length and intron number columns are added.
# The reformated results are output into two new files within the new RESULTS directory.
# To avoid confusion, the K_noncoding_out.tab file is deleted.	  
#
########################################################################################################################################################

package provide INTRONS17_Kgene 1.0

proc INTRONS17_Kgene {} {
	
	puts "-----------------------------------------------------------"
	puts "17         CALCULATING SUBSTITUTION RATES - GENES          \n"

	# Make variables global
	global contents
	
    # Move to Intron_Data directory and do all work in this directory
	cd Intron_Data

	
	# Create new RESULTS directory
	if {![file exists RESULTS]} {
		puts "No RESULTS directory was found"
		exec mkdir RESULTS
		puts "New RESTULTS directory was created"
	}
	
	puts "Preparation for Knc.pl script in alignment files directory"
	# Move perl Knc.pl script into the directory containing the final data set
	exec cp ../mmREPLICATION_tcl_library/Knc.pl ./CONCATENATE_genes/Knc.pl
	
	# Move to CONCATENATE_chromosomes directory
	cd CONCATENATE_genes

	# Remove any K_noncoding_out.tab files already in existance to prevent new results appending to current file
	if {[file exists K_noncoding_out.tab]} {
		exec rm K_noncoding_out.tab
	}
    
	puts "Executing Knc.pl script"
	# Execute Martins Knc.pl script to calculate substitution rates
	exec perl Knc.pl
	puts "Substitution rates calculated"
	
	# Remove Knc.pl script from directory
	exec rm Knc.pl
    
	# Move output K_noncoding_out.tab file to RESULTS directory
	exec mv K_noncoding_out.tab ../RESULTS/K_noncoding_out.tab

	# Return from CONCATENATE_chromosomes to Intron_Data directory
	cd ..
	puts "Tidied Knc.pl and output file from alignment files directory"


	puts "Reformating output results file"
	# Move to RESULTS directory
	cd RESULTS

	# Create a new output file to store results in
	set outTXT [open K_gene.txt w]
	set outXLS [open K_gene.xls w]

	puts $outTXT "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start\tMouse_End\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	puts $outXLS "Ortholog\tMouse_Chromosome\tRat_Chromosome\tMouse_Refseq\tRat_Refseq\tIntrons_Concatenated\tMouse_Start\tMouse_End\tRat_Start\tRat_End\tAlignment_Length\tN\tK_JC\tK_Kimura\tK_TN\tK_TK\traw_subst\traw_subst_per_site\tGC"
	
	# Reformat each line to include mouse chromosome, alignment length and number of introns rather than file or alignment names
	openfile K_noncoding_out.tab
	set contents [split $contents \n]
	set contents [lrange $contents 1 end-1]
	foreach dataline $contents {
		
		set alignmentfilename [lindex $dataline 0]
		set mousename [lindex $dataline 1]
		set ratname [lindex $dataline 2]

		regexp {Concat_Orth-([0-9]+)-Chr-[\w]+-Length-([0-9]+)\.nuc} $alignmentfilename all ortholog alignmentlength
		regexp {>mm:(NM_[\w]+):chr([\w]+):#Introns-([0-9]+):([0-9]+)-([0-9]+)} $mousename all mouserefseq mousechromosome numberofintrons mousestart mouseend
		regexp {>rn:(NM_[\w]+):chr([\w]+):#Introns-([0-9]+):([0-9]+)-([0-9]+)} $ratname all ratrefseq ratchromosome numberofintrons ratstart ratend
		
		# Only output results with no -1 error values
		
		if {[lindex $dataline 4] != -1 && [lindex $dataline 5] != -1 && [lindex $dataline 6] != -1 && [lindex $dataline 7] != -1} {
			puts $outTXT "$ortholog\t$mousechromosome\t$ratchromosome\t$mouserefseq\t$ratrefseq\t$numberofintrons\t$mousestart\t$mouseend\t$ratstart\t$ratend\t$alignmentlength\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]"
			puts $outXLS "$ortholog\t$mousechromosome\t$ratchromosome\t$mouserefseq\t$ratrefseq\t$numberofintrons\t$mousestart\t$mouseend\t$ratstart\t$ratend\t$alignmentlength\t[lindex $dataline 3]\t[lindex $dataline 4]\t[lindex $dataline 5]\t[lindex $dataline 6]\t[lindex $dataline 7]\t[lindex $dataline 8]\t[lindex $dataline 9]\t[lindex $dataline 10]"
		}
		
		unset alignmentfilename mousename ratname
		unset ortholog alignmentlength
		unset mouserefseq mousechromosome numberofintrons mousestart mouseend
		unset ratrefseq ratchromosome ratstart ratend
	}
	unset dataline contents

	# Close new results file
	close $outTXT
	close $outXLS
	unset outTXT outXLS
	
	# Remove previous K_noncoding_out.tab file
	exec rm K_noncoding_out.tab
	
	# Return from RESULTS to Intron_Data directory
	cd ..
	# Return from Intron_Data directory to main directory
	cd ..

	puts "\nVariables open in INTRONS17_Kgene : [info local]"

return
}